//	Библиотека работы arduino с HEXAPOD http://iarduino.ru/shop/Mehanika/robot-hexapod-shestinogiy.html
//
//	Подключение библиотеки:
//	#include <iarduino_Hexapod.h>
//	iarduino_Hexapod IXP([кнопка A, кнопка B, светодиод]); Объявляем переменную IXP, для работы с Hexapod, указывая номера цифровых выводов к которым подключены: кнопка A, кнопка B, светодиод
//	
//	В библиотеке реализованы 6 функций:
//	
//	Функция begin             Назначение:            Инициализация Hexapod
//	                          Синтаксис:             begin(тип_сервоприводов);
//	                          Параметры:             указывается один из следующих параметров:
//	                                                 SERVO_SG90        http://iarduino.ru/shop/Mehanika/mikroservoprivod---1-6-kg-sm.html
//	                                                 SERVO_MG90        http://iarduino.ru/shop/Mehanika/mg995r-pro-servo.html
//	                                                 SERVO_MG996R      http://iarduino.ru/shop/Mehanika/mg995r-pro-servo.html
//	                                                 SERVO_FutabaS3003 http://iarduino.ru/shop/Mehanika/futaba-s3003.html
//	                          Возвращаемые значения: Нет
//	                          Примечание:            Вызывается 1 раз в коде setup.
//	
//	Функция go                Назначение:            Движение Hexapod
//	                          Синтаксис:             1 вариант: go(X [, Y [, Z]]);
//	                                                 2 вариант: go(команда [, параметр_команды]);
//	                          Параметры:             X от -10 до +10   движение влево (-10) / вправо (+10)
//	                                                 Y от -10 до +10   движение назад (-10) / вперёд (+10) - (необязательный параметр)
//	                                                 Z от -10 до +10   движение вниз  (-10) / вверх  (+10) - (необязательный параметр)
//	                                                 параметр_команды  значение от 0 до 10 определяющее скорость или уровень выполнения команды - (необязательный параметр)
//	                                                 команда           может принимать одно из следующих значений:
//	                                                 GO_ON             движение вперед
//	                                                 GO_BACK           движение назад
//	                                                 GO_LEFT           поворот  влево
//	                                                 GO_RIGHT          поворот  вправо
//	                                                 GO_ON_LEFT        движение вперед и поворот влево
//	                                                 GO_ON_RIGHT       движение вперед и поворот вправо
//	                                                 GO_BACK_LEFT      движение назад  и поворот влево
//	                                                 GO_BACK_RIGHT     движение назад  и поворот вправо
//	                                                 COM_STOP          команда  стоп
//	                                                 COM_SIT           команда  сидеть
//	                                                 COM_NOD           команда  поклон
//	                                                 COM_UP            команда  встать
//	                                                 COM_LAY           команда  лежать
//	                                                 COM_TEST          команда  тест
//	                                                 COM_SLEEP         команда  спать
//	                                                 COM_EMPTY         команда  пустая
//	                          Возвращаемые значения: Нет
//	
//	Функция set_motion        Назначение:            Движение конечностями Hexapod
//	                          Синтаксис:             set_motion(№_конечности , №_сустава , положение_сустава);
//	                          Параметры:             №_конечности      от 1 до 6    или SERVO_ALL для всех конечностей сразу
//	                                                 №_сустава         от 1 до 2    1-вертикальный / 2-горизонтальный
//	                                                 положение_сустава от 0 до 100  0-внизу или сзади / 100 вверху или спереди
//	                          Возвращаемые значения: Нет
//	
//	Функция set_motionA       Назначение:            Движение конечностями Hexapod по градусу
//	                          Синтаксис:             set_motionA(№_конечности , градус_движения);
//	                          Параметры:             №_конечности      от 1 до 6    или SERVO_ALL для всех конечностей сразу
//	                                                 градус_движения   от 1 до 360  см. примечание
//	                          Возвращаемые значения: Нет
//	                          Примечание:            Представьте себе, как движется конечность ...
//	                                                 Она повторяет одни и те же движения (иногда с разным размахом): проходит внизу спереди назад, поднимается вверх, сверху быстрее проходит вперёд, опускается вниз, и всё повторяется.
//	                                                 Если представить весь этот круг ввиде 360°, то каждое положение конечности, можно представить ввиде числа от 1 до 360, это и есть градус_движения.
//	
//	Функция calibration       Назначение:            калибровка суставов конечностей Hexapod
//	                          Синтаксис:             calibration();
//	                          Параметры:             Нет
//	                          Возвращаемые значения: Нет
//	                          Примечание:            Вызывается 1 раз в коде loop. Сама перехыатывает нажатие кнопок A и B
//	
//	Функция calibration_reset Назначение:            сброс ранее сохранённых значений калибровки Hexapod
//	                          Синтаксис:             calibration_reset();
//	                          Параметры:             Нет
//	                          Возвращаемые значения: Нет
//	                          Примечание:            Вызывается 1 раз в коде setup. Может понадобится для тестовых целей
//	
//	В библиотеке доступна 1 переменная:
//	
//	uint8_t                   walking                тип походки (по умолчанию = 2)
//	                                                 0 - все конечности движутся одинаково, похоже на плавание брасом
//	                                                 1 - конечности движутся с задержкой на 1/6 в следующем порядке: 1,2,3,4,5,6
//	                                                 2 - конечности движутся с задержкой на 1/6 в следующем порядке: 1,3,5,2,4,6
//	                                                 3 - конечности движутся с задержкой на 1/6 в следующем порядке: 1,4,5,2,3,6

#ifndef iarduino_Hexapod_h
#define iarduino_Hexapod_h
#include <iarduino_MultiServo.h>
#include <EEPROM.h>
#include "Servo.h"

#if defined(ARDUINO) && (ARDUINO >= 100)
#include <Arduino.h>
#else
#include <WProgram.h>
#endif

// команды для функции go
#define	COM_SLEEP			100	//	команда  спать
#define	GO_ON				101	//	движение вперед
#define	GO_BACK				102	//	движение назад
#define	GO_LEFT				103	//	поворот  влево
#define	GO_RIGHT			104	//	поворот  вправо
#define	GO_ON_LEFT			105	//	движение вперед и поворот влево
#define	GO_ON_RIGHT			106	//	движение вперед и поворот вправо
#define	GO_BACK_LEFT		107	//	движение назад  и поворот влево
#define	GO_BACK_RIGHT		108	//	движение назад  и поворот вправо
#define	COM_STOP			109	//	команда  стоп
#define	COM_SIT				110	//	команда  сидеть
#define	COM_NOD				111	//	команда  поклон
#define	COM_UP				112	//	команда  встать
#define	COM_LAY				113	//	команда  лежать
#define	COM_TEST			114	//	команда  тест
#define	COM_EMPTY			255	//	команда  пустая

#define	SERVO_FREE			255	//	освободить сустав (свободный ход)

// команды для функции position

// команды для функции motion

class iarduino_Hexapod{
	public:			iarduino_Hexapod	(uint8_t=255,uint8_t=255,uint8_t=13);	//										(кнопка A, кнопка B, светодиод)	
	/**	пользовательские функции **/
		void		begin						(uint8_t);						//	инициализация HEXAPOD				(тип сервоприводов)
		void		calibration					();								//	калибровка суставов конечностей		(без параметров)
		void		calibration_reset			();								//	сброс калибровки					(без параметров)
		void		go							(int=255,int=255,int=255);		//	движение							(-10 лево / +10 право , -10 назад / +10 вперед , -10 низ / +10 верх)
																				//										(команда , скорость / параметр)
		void		set_motion					(uint8_t,uint8_t,uint8_t);		//	движение одной/всеми конечностями	(номер конечности: 1-6 / 255 , номер сустава: 1=V / 2=H , положение сустава: внизу-сзади 0...50...100 вверху-спереди / 255)
//		uint8_t		get_motion					(uint8_t,uint8_t);				//	текущее положение одной конечности	(номер конечности: 1-6       , номер сустава: 1=V / 2=H)
		void		set_motionA					(uint8_t,int,bool=0,bool=0);	//	движение одной/всеми конечностями	(номер конечности: 1-6 / 255 , градус движения: 1-360 , учитывать ограничение по вертикали: iXP_limit_Ver % , учитывать ограничение по горизонтали: iXP_limit_Hor % )
//		uint8_t		get_motionA					(uint8_t,uint8_t);				//	текущее положение одной конечности	(номер конечности: 1-6       , номер сустава: 1=V / 2=H)
	/**	пользовательские переменные **/
		uint8_t		walking						= 2;							//	тип походки
	private:
	/**	внутренние функции **/
		void		iXP_func_Go					(uint8_t,uint8_t);				//	движение по константам				(GO_ON вперед / GO_BACK назад / ... , скорость / параметр)
		uint8_t		iXP_func_JointPin			(uint8_t,uint8_t);				//	определение номера вывода			(номер конечности 1-6 , номер сустава 1=V / 2=H)
		uint8_t		iXP_func_AngleVer			(uint8_t,uint8_t);				//	определение угла по вертикали		(номер конечности 1-6 , положение вертикального   сустава: внизу 0...50...100 вверху)
		uint8_t		iXP_func_AngleHor			(uint8_t,uint8_t);				//	определение угла по горизонтали		(номер конечности 1-6 , положение горизонтального сустава: сзади 0...50...100 спереди)
		bool		iXP_func_ReadAngleCenter	();								//	чтение сохранённых значений центрального угла, для каждого сустава, каждой конечности
		void		iXP_func_CalibrationLed		();								//	управление светодиодом во время калибровки
	/**	внутренние переменные **/
					iarduino_MultiServo iXP_MSS;
		#include	"Settings.h"												//	настройки суставов конечностей
		uint8_t		iXP_angle_Hor_Center[6]		= {95,95,95,95,95,95};			//	центральный угол горизонтального сустава для каждой конечности
		uint8_t		iXP_angle_Ver_Center[6]		= {95,80,95,80,95,80};			//	центральный угол вертикального   сустава для каждой конечности
		uint8_t		iXP_angle_Odd_Calibration	= 30;							//	верхний угол при установке  нечетных конечностей (во время калибровки)
		uint8_t		iXP_angle_Evn_Calibration	= 150;							//	верхний угол при установке    четных конечностей (во время калибровки)
		uint8_t		iXP_limit_Ver				= 50;							//	ограничение по вертикали   0 - 100%
		uint8_t		iXP_limit_Hor_Odd			= 0;							//	ограничение по горизонтали 0 - 100% (для нечетных конечностей)
		uint8_t		iXP_limit_Hor_Evn			= 0;							//	ограничение по горизонтали 0 - 100% (для   четных конечностей)
		int16_t		iXP_gogo_360				= 0;							//	текущий градус движения    0 - 360
		bool		iXP_flag_Calibration		= false;						//	флаг указывающий о том, что калибровка была произведена
		uint8_t		iXP_step_Calibration		= 0;							//	шаг/этап калибровки
		uint8_t		iXP_pins_ButtonA;											//	номера вывода к которому подключена кнопка A
		uint8_t		iXP_time_ButtonA			= 0;							//	время удержания кнопки A
		uint8_t		iXP_pins_ButtonB;											//	номера вывода к которому подключена кнопка B
		uint8_t		iXP_time_ButtonB			= 0;							//	время удержания кнопки B
		uint8_t		iXP_pins_LED;												//	номера вывода к которому подключен светодиод
	unsigned long	iXP_time_LED				= 0;							//	время включения светодиода в мс
		uint16_t	iXP_time_ButtonDelay		= 2000;							//	время удержания кнопок для перехода по режимам калибровки (в мс)
		uint16_t	iXP_addr_EEPROM				= 497;							//	адрес (от 0 до 497) первой, из 13, ячеек памяти EEPROM в которых хранятся значения горизонтальных и вертикальных настроек
};

#endif